@@ -126,25 +126,23 @@ addToLibrary({
     readFile(path, opts = {}) {
       opts.encoding = opts.encoding || 'binary';
       if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
-        throw new Error('Invalid encoding type "' + opts.encoding + '"');
+        throw new Error(`Invalid encoding type "${opts.encoding}"`);
       }
 
+      var buf, length;
       // Copy the file into a JS buffer on the heap.
-      var sp = stackSave();
-      var buf = __wasmfs_read_file(stringToUTF8OnStack(path));
-      stackRestore(sp);
-
-      // The signed integer length resides in the first 8 bytes of the buffer.
-      var length = {{{ makeGetValue('buf', '0', 'i53') }}};
+      withStackSave(() => {
+        var bufPtr = stackAlloc({{{ POINTER_SIZE }}});
+        var sizePtr = stackAlloc({{{ POINTER_SIZE }}});
+        FS.handleError(-__wasmfs_read_file(stringToUTF8OnStack(path), bufPtr, sizePtr));
+        buf = {{{ makeGetValue('bufPtr', '0', '*') }}};
+        length = {{{ makeGetValue('sizePtr', '0', 'i53') }}};
+      });
 
       // Default return type is binary.
       // The buffer contents exist 8 bytes after the returned pointer.
-      var ret = new Uint8Array(HEAPU8.subarray(buf + 8, buf + 8 + length));
-      if (opts.encoding === 'utf8') {
-        ret = UTF8ArrayToString(ret);
-      }
-
-      return ret;
+      var ret = new Uint8Array(HEAPU8.subarray(buf, buf + length));
+      return opts.encoding === 'utf8' ? UTF8ArrayToString(ret) : ret;
     },
 #endif
 
@@ -182,10 +180,7 @@ addToLibrary({
     create: (path, mode) => FS_create(path, mode),
     close: (stream) => FS.handleError(-__wasmfs_close(stream.fd)),
     unlink: (path) => FS_unlink(path),
-    chdir: (path) => withStackSave(() => {
-      var buffer = stringToUTF8OnStack(path);
-      return __wasmfs_chdir(buffer);
-    }),
+    chdir: (path) => withStackSave(() => __wasmfs_chdir(stringToUTF8OnStack(path))),
     read(stream, buffer, offset, length, position) {
       var seeking = typeof position != 'undefined';
 
@@ -197,14 +192,12 @@ addToLibrary({
       } else {
         bytesRead = __wasmfs_read(stream.fd, dataBuffer, length);
       }
-      bytesRead = FS.handleError(bytesRead);
-
-      for (var i = 0; i < length; i++) {
-        buffer[offset + i] = {{{ makeGetValue('dataBuffer', 'i', 'i8')}}}
+      if (bytesRead > 0) {
+        buffer.set(HEAPU8.subarray(dataBuffer, dataBuffer + bytesRead), offset);
       }
 
       _free(dataBuffer);
-      return bytesRead;
+      return FS.handleError(bytesRead);
     },
     // Note that canOwn is an optimization that we ignore for now in WasmFS.
     write(stream, buffer, offset, length, position, canOwn) {
@@ -221,13 +214,8 @@ addToLibrary({
       } else {
         bytesRead = __wasmfs_write(stream.fd, dataBuffer, length);
       }
-      bytesRead = FS.handleError(bytesRead);
       _free(dataBuffer);
-
-      return bytesRead;
-    },
-    allocate(stream, offset, length) {
-      return FS.handleError(__wasmfs_allocate(stream.fd, {{{ splitI64('offset') }}}, {{{ splitI64('length') }}}));
+      return FS.handleError(bytesRead);
     },
     writeFile: (path, data) => FS_writeFile(path, data),
     mmap: (stream, length, offset, prot, flags) => {
@@ -247,8 +235,12 @@ addToLibrary({
       __wasmfs_symlink(stringToUTF8OnStack(target), stringToUTF8OnStack(linkpath))
     )),
     readlink(path) {
-      var readBuffer = FS.handleError(withStackSave(() => __wasmfs_readlink(stringToUTF8OnStack(path))));
-      return UTF8ToString(readBuffer);
+      return withStackSave(() => {
+        var bufPtr = stackAlloc({{{ POINTER_SIZE }}});
+        FS.handleError(__wasmfs_readlink(stringToUTF8OnStack(path), bufPtr));
+        var readBuffer = {{{ makeGetValue('bufPtr', '0', '*') }}};
+        return UTF8ToString(readBuffer);
+      });
     },
     statBufToObject(statBuf) {
       // i53/u53 are enough for times and ino in practice.
@@ -268,25 +260,19 @@ addToLibrary({
           ino: {{{ makeGetValue('statBuf', C_STRUCTS.stat.st_ino, "u53") }}}
       }
     },
-    stat(path) {
-      var statBuf = _malloc({{{ C_STRUCTS.stat.__size__ }}});
-      FS.handleError(withStackSave(() =>
-        __wasmfs_stat(stringToUTF8OnStack(path), statBuf)
-      ));
-      var stats = FS.statBufToObject(statBuf);
-      _free(statBuf);
-
-      return stats;
+    stat(path) { 
+      return withStackSave(() => {
+        var statBuf = stackAlloc({{{ C_STRUCTS.stat.__size__ }}});
+        FS.handleError(__wasmfs_stat(stringToUTF8OnStack(path), statBuf));
+        return FS.statBufToObject(statBuf);
+      });
     },
     lstat(path) {
-      var statBuf = _malloc({{{ C_STRUCTS.stat.__size__ }}});
-      FS.handleError(withStackSave(() =>
-        __wasmfs_lstat(stringToUTF8OnStack(path), statBuf)
-      ));
-      var stats = FS.statBufToObject(statBuf);
-      _free(statBuf);
-
-      return stats;
+      return withStackSave(() => {
+        var statBuf = stackAlloc({{{ C_STRUCTS.stat.__size__ }}});
+        FS.handleError(__wasmfs_lstat(stringToUTF8OnStack(path), statBuf));
+        return FS.statBufToObject(statBuf);
+      });
     },
     chmod(path, mode) {
       return FS.handleError(withStackSave(() => {
@@ -351,7 +337,7 @@ addToLibrary({
       return FS.handleError(withStackSave(() => __wasmfs_mount(stringToUTF8OnStack(mountpoint), backendPointer)));
     },
     unmount: (mountpoint) => (
-      FS.handleError(withStackSave(() => __wasmfs_unmount(stringToUTF8OnStack(mountpoint))))
+      FS.handleError(withStackSave(() => _wasmfs_unmount(stringToUTF8OnStack(mountpoint))))
     ),
     // TODO: lookup
     mknod: (path, mode, dev) => FS_mknod(path, mode, dev),
@@ -554,9 +540,10 @@ addToLibrary({
   $FS_mkdirTree: (path, mode) => {
     var dirs = path.split('/');
     var d = '';
-    for (var i = 0; i < dirs.length; ++i) {
-      if (!dirs[i]) continue;
-      d += '/' + dirs[i];
+    for (var dir of dirs) {
+      if (!dir) continue;
+      if (d || PATH.isAbs(path)) d += '/';
+      d += dir;
       try {
         FS_mkdir(d, mode);
       } catch(e) {
